from bs4 import BeautifulSoup
import requests

def get_classes_of ( metabolite='HMDB0000267', path=None ):
    url = ['http://www.hmdb.ca/metabolites/','.xml']
    response = requests.get(metabolite.join(url))
    data = BeautifulSoup ( response.content )
    return ( data.findAll(['super_class','class','sub_class']) )



from lxml import etree

def get_accession( child, common_name ) :
    accessions=[] ; names=[] ; hmdb_ids=[]
    for element in child.iter():
        if element.text is None:
            continue
        if 'name' in element.tag or 'synonym' in element.tag :
            names.append(element.text.replace('\n',''))
        if 'accession' in element.tag:
            acc = element.text.replace('\n','').replace(' ','')
            if len(acc)>0:
                accessions.append(acc)
    bHaveMatch=False
    if 'list' in str(type(common_name)) :
        cey = [ cn for cn in common_name if (cn in set(names)) ]
        if len(cey)>0:
            bHaveMatch=True
            cey = cey[0]
    else:
        cey = common_name
        if cey in set(names):
            bHaveMatch=True
    if bHaveMatch:
        return(accessions,names,cey)
    else :
        return([],[],[])

if __name__=='__main__':

    translate_these = ["Phenylalanine (13C9) [M+H]+", "Phenylalanine (13C9) [M+H-HCOOH]+", "Caffeine (trimethyl-13C3) [M+H]+", "Cholic Acid (2,2,4,4-D4) [M+H-2*H2O]+", "Cholic Acid (2,2,4,4-D4) [M+H-3*H2O]+", "Cholic Acid (2,2,4,4-D4) [M+NH4]+", "prolylhydroxyproline", "kynurenate", "indolepropionate", "indolelactate", "5-methylthioadenosine (MTA)", "phenylacetylglutamine", "tryptophan betaine", "thyroxine", "creatinine", "creatine", "methionine", "5-oxoproline", "pipecolate", "histidine", "3-Methylcrotonylglycine", "homoarginine", "dimethylarginine (ADMA + SDMA)", "phenylalanine", "acisoga", "Propionylcarnitine (C3)", "Succinylcarnitine (C4)", "Isobutyrylcarnitine (C4)", "Butyrylcarnitine (C4)", "Hydroxybutyrylcarnitine (C4:0-OH)", "Tiglyl carnitine (C5:1)", "Glutaroylcarnitine (C5)", "2-methylbutyroylcarnitine (C5)", "Isovalerylcarnitine (C5)", "Valerylcarnitine (C5)", "Hydroxyisovaleroylcarnitine", "3-methylglutarylcarnitine", "3-methylglutarylcarnitine (2)", "Hexanoylcarnitine (C6)", "Octenoyl-carnitine (C8:1)", "Octanoyl-carnitine (C8)", "cis-4-decenoyl carnitine (C10:1)", "Decanoylcarnitine (C10)", "Dodecenoyl-carnitine (C12:1)", "Laurylcarnitine (C12)", "Hydroxylauroyl-carnitine (C12:0-OH)", "Tetradecdienoyl-carnitine (C14:2)", "Myristoleoylcarnitine (C14:1)", "Myristoylcarnitine (C14)", "Hydroxymyristoyl-carnitine (C14:0-OH)", "Hexadecdienoyl-carnitine (C16:2)", "Hexadecenoyl-carnitine (C16:1)", "Palmitoylcarnitine (C16)", "Linoleneoyl-carnitine (C18:3)", "Linoleoylcarnitine (C18:2)", "Oleoylcarnitine (C18:1)", "Hydroxyoctadecenoyl-carnitine (C18:1-OH))", "Stearoylcarnitine (C18)", "Eicosadieneoyl-carnitine (C20:2)", "Eicoseneoyl-carnitine (C20:1)", "Cervonyl-carnitine (C22:6)", "Carnitine", "Acetylcarnitine (C2)", "trigonelline (N'-methylnicotinate)", "Betaine", "I-urobilinogen", "bilirubin", "gamma-CEHC", "pyridoxate", "pantothenate (Vitamin B5)", "L-urobilin", "biliverdin", "alpha-CEHC", "Fructosyl valine", "Fructosyl isoleucine", "Fructosyl leucine", "1-palmitoylplasmenylethanolamine*", "2-palmitoyl-GPE* (16:0)*", "1-palmitoyl-GPE (16:0)", "1-myristoyl-GPC (14:0)", "PAF(12:0)", "1-linoleoyl-GPE (18:2)*", "1-stearoylplasmenylethanolamine*", "1-stearoyl-GPE (18:0)", "1-pentadecanoylglycerophosphocholine (15:0)*", "PAF (14:1)", "palmitoleoyl-GPC* (16:1)*", "palmitoyl-GPC (16:0)", "PAF (14:0)", "1-eicosapentaenoylglycerophosphoethanolamine*", "2-arachidonoyl-GPE* (20:4)*", "1-heptadecanoyl-GPC (17:0)", "1-margaroylglycerophosphoethanolamine*", "1-linolenoylglycerophosphocholine (18:3n3)*", "oleoyl-GPC (18:1)", "PAF (16:1)", "PAF (16:0)", "stearoyl-GPC (18:0)", "1-nonadecanoylglycerophosphocholine(19:0)", "1-eicosapentaenoylglycerophosphocholine (20:5n3)*", "1-arachidonoyl-GPC* (20:4)*", "1-eicosatrienoyl-GPC* (20:3)*", "1-eicosadienoyl-GPC* (20:2)*", "2-eicosenoylglycerophosphocholine(20:1n9)*", "1-eicosenoylglycerophosphocholine (20:1n9)*", "1-arachidoyl-GPC (20:0)", "1-docosahexaenoyl-GPC* (22:6)*", "1-docosapentaenoyl-GPC* (22:5n6)*", "urate", "7-methylguanine", "2'-O-methylguanosine", "N6-succinyladenosine", "hypoxanthine", "N2,N2-dimethylguanosine", "uracil", "1-methylguanosine", "pyroglutamylvaline", "gamma-glutamylglutamine", "aspartylleucine", "gamma-glutamylmethionine", "N-acetylcarnosine", "cyclo(leu-pro)", "gamma-glutamylleucine", "phenylalanylalanine", "isoleucylleucine", "phenylalanylglutamate", "gamma-glutamyltyrosine", "phenylalanylleucine", "gamma-glutamyltryptophan", "phenylalanylphenylalanine", "phenylalanyltryptophan", "cis-Cyclo[L-ala-L-Pro]", "gamma-glutamylalanine", "leucylglycine", "11-dehydrocorticosterone", "7-HOCA", "cortisol", "20beta-dihydrocortisone", "EDTA", "2-oxindole-3-acetate", "cinnamoylglycine", "1-methylxanthine", "1-methylurate", "7-methylurate", "theobromine", "1,3-dimethylurate", "caffeine", "4-acetaminophen sulfate", "2-methoxyacetaminophen sulfate*", "hippurate", "3-hydroxyhippurate", "Phenylalanine (13C9) [M-H]-", "Caffeic Acid (13C9) [M-H]-", "Cholic Acid (2,2,4,4-D4) [M-H]-", "Arachidonic Acid (C20:4) (D8) [M-H]-", "Ursodeoxycholic acid (?)", "Deoxycholcholic acid", "Chenodeoxycholic acid", "Cholic acid", "Lithocholic acid glycine conjugate", "Glycodeoxycholcholic acid", "Chenodeoxycholic acid glycine conjugate", "Glycoursodeoxycholic acid", "Hyocholic acid glycine conjugate (?)", "Glycocholic acid", "Taurochenodesoxycholic acid", "Taurodeoxycholic acid", "Taurocholcholic acid (?)", "3,7-Dihydroxy-5-cholestenoic acid", "Sebacate (decanedioate)", "3-Hydroxydecanoate", "2-Hydroxydecanoate", "3-Carboxy-4-methyl-5-propyl-2-furanpropanoate (CMPF)", "Dodecanedioate (C12)", "3-Hydroxylaurate", "Tetradecanedioate (C14)", "Myristate (14:0)", "3-Hydroxymyristate", "Hydroxymyristate", "Palmitoleate (16:1)", "Hexadecanedioate (C16)", "Palmitate (16:0)", "2-Hydroxypalmitate", "3-Hydroxypalmitate", "10-Heptadecanoate (17:1)", "Linolenate (18:3)", "Linoleate (18:2)", "13-HODE + 9-HODE", "13-HODE + 9-HODE_b", "Oleate (18:1)", "Octadecanedioate (C18)", "Stearate (18:0)", "Hydroxystearate", "2-Hydroxystearate", "Eicosapentaenoate (EPA; 20:5)", "Arachidonate (20:4)", "Mead acid (20:3)", "Mead acid (20:3)_b", "Eicosanodioate", "1-Linoleoylglycerol (18:2)", "1-Oleoylglycerol (18:1) (?)", "Docosahexaenoate (DHA; 22:6)", "Docosapentaenoate (n6 DPA; 22:5_b)", "Docosapentaenoate (DPA; 22:5)", "1-Arachidonoylglycerol", "3-Hydroxybutyrate (BHBA)", "2-hydroxybutyrate", "2-Methylglutaric acid", "2-Hydroxyhexanoate", "5-hydroxyhexanoate", "Suberate (octanedioate)", "3-Hydroxyoctanoate", "2-Hydroxyoctanoate", "Azelate (nonanedioate; C9)", "3-Hydroxy-2-ethylpropionate", "Ethylmalonic acid", "Methylsuccinic acid", "Ketoleucine", "alpha-hydroxyisocaporic acid", "3-Methylcrotonylglycine", "N-acetyl-leucine", "Sphingosine 1-P", "testosterone sulfate", "16a-hydroxy DHEA 3-sulfate", "4-androsten-3beta,17beta-diol monosulfate (1)", "4-androsten-3alpha,17alpha-diol monosulfate (2)", "4-androsten-3alpha,17alpha-diol monosulfate (3)", "11-dehydrocorticosterone", "cortisol", "21-hydroxypregnenolone monosulfate (1)", "pregnen-diol disulfate* ?", "5alpha-pregnan-3(alpha or beta),20beta-diol disulfate ?", "Drospirenone", "7-HOCA", "pregnanediol-3-glucuronide", "dihydroferulic acid", "thymol sulfate", "cinnamoylglycine", "1-methylxanthine", "1-methylurate", "catechol sulfate", "5-acetylamino-6-amino-3-methyluracil", "quinate", "salicylate", "paraxanthine", "1,3-dimethylurate", "2-ethylphenylsulfate", "4-acetaminophen sulfate", "4-hydroxycoumarin", "hippurate", "3-hydroxyhippurate", "2-hydroxyhippurate (salicylurate)", "SUCCINIC ACID-D4-2TMS   1", "SALICYLIC ACID-D4-2TMS   1", "13C5-15N-PYROGLUTAMIC ACID,N,O-TMS (IS-glutamate-deriv)   1", "ALPHA-KETOGLUTARATE-13C4-MEOX 2TMS   1", "PUTRESCINE-D4-4TMS   1", "MYRISTIC ACID-13C3-TMS   1", "HEXADECANOIC ACID-13C4-TMS   1", "Octadecanoic acid methyl ester, n-   1", "SUCROSE-13C12-8TMS   1", "ALANINE-N,O-TMS   1", "5,6-Dihydrouracil-2TMS   1", "3-HYDROXYBUTYRIC ACID-2TMS   1", "UREA-3TMS_1   1", "VALINE,N,O-TMS   1", "Urea, 2TMS derivative   1", "PHOSPHATE-FRAGMENT   1", "ISOLEUCINE, N,O-TMS   1", "PROLINE,N,O-TMS   1", "GLYCINE,N,N,O-TMS   1", "W161_N/A   1", "SUCCINIC ACID 2TMS   1", "GLYCERIC ACID-3TMS   1  M/z294", "FUMARIC ACID-2TMS   1", "SERINE,N,O,O-TMS   1", "THREONINE,N,O,O-TMS   1", "W169_N/A   1", "W170_N/A like  Amino Acid   1", "Iminodiacetic acid (2TMS)   1", "Aminomalonic acid, tris(trimethylsilyl)-   1", "W172_N/A like Xylulose   1", "MALIC ACID-3TMS   1  M/z336", "Asparagine [-H2O] (2TMS)   1", "SALICYLIC ACID-2TMS   1", "ERYTHRITOL-4TMS   1", "N/A like Amine   1", "PYROGLUTAMIC ACID,N,O-TMS   1", "W177_N/A like composite of Hydroxyproline   1", "L-Aspartic acid, 3TMS derivative   1", "W378_M001212_A155003-101-xxx_NA_1541,53_PRED_VAR5_ALK_similar to Ditertbutylphenol (1TMS)   1", "L-Threonic acid, tris(trimethylsilyl) ether, trimethylsilyl ester   1", "CREATININE-3TMS   1", "W179_N/A like Threonic acid   1", "W304_N/A M000000_A156015-101-xxx_NA_1558,59_PRED_VAR5_ALK_Unknown#bth-pae-039   1", "ALPHA-KETOGLUTARIC ACID-MEOX 2TMS   1", "GLUTAMINE,N,N,N,O-TMS_3   1", "ORNITHINE,N,N,O-TMS_1   1", "Ibuprofen-1TMS   1", "PHENYLALANINE,N,O-TMS   1", "ACETAMINOPHEN-2TMS   1", "W186_N/A like Amine   1", "XYLOSE-MEOX 4TMS_1   1", "DODECANOIC  ACID-TMS   1", "ASPARAGINE,N,N,O-TMS   1", "W189_N/A like Deoxy-ribose isomer   1", "LYSINE,N,N,O-TMS_1   1", "W191_N/A like Succinylacetone   1", "2-AMINO-ADIPIC ACID, N,O,O-TMS   1", "RIBITOL-5TMS   1", "W195_N/A   1", "GLYCEROL-3-PHOSPHATE-4TMS   1", "W197_N/A like Furanose   1", "GLUTAMINE,N,N,O-TMS_1   1", "HIPPURIC ACID-2TMS   1", "ORNITHINE,N,N,N,O-TMS_2   1", "ARGININE-3TMS   1  M/z256", "1,5-ANHYDRO-D-GLUCITOL-4TMS   1", "FRUCTOSE-MEOX 5TMS_1   1", "MANNOSE-MEOX 5TMS_1   1", "W204_N/A like  Amino Acid   1", "GLUCOSE-MEOX 5TMS_1   1", "MANNITOL-6TMS   1", "TYROSINE,N,O,O-TMS   1", "INOSITOL, chiro-6TMS   1", "W205_N/A like Disaccharide   1", "W207_N/A like Disaccharide   1", "beta-D-Glucopyranose, 5TMS derivative   1", "PALMITOLEIC ACID-TMS   1", "W213_N/A like Hexose Pyranose   1", "HEXADECANOIC ACID-TMS   1", "W145_N/A M000000_A208005-101-xxx_NA_2073,95_PRED_VAR5_ALK_NA   1", "NAPROXEN-TMS   1", "INOSITOL, myo-6TMS   1", "URIC ACID-N,O,O,O-TMS   1", "W218_N/A like EDTA   1", "W221_N/A   1", "W223_N/A   1", "W225_N/A like EDTA   1", "W146_N/A M000000_A221004-101-xxx_NA_2197,89_PRED_VAR5_ALK_NA   1", "TRYPTOPHAN,N,N,O-TMS   1", "OCTADECENOIC ACID,-9-(Z)-TMS   1", "OCTADECANOIC ACID-TMS   1", "W147_N/A M000000_A226001-101-xxx_NA_2244,52_PRED_VAR5_ALK_NA   1", "CYSTINE,N,N,O,O-TMS   1", "W235_N/A   1", "Inositol-phosphate (7TMS)   1", "EDTA-4TMS peak 2 major RI 2407 M/z 290    1", "XYLOBIOSE-MEOX 6TMS_2   1", "Phthalic acid, di(2-propylpentyl) ester   1", "1-Monopalmitin, 2TMS derivative   1", "SUCROSE-8TMS   1", "W309_N/A like Sterol   1", "W244_N/A   1", "W359_N/A   1", "MALTOSE-MEOX 8TMS_1   1  M/z362", "W246_N/A Drug (one individual several timepoints)   1", "W310_N/A like Monostearin   1", "Acetaminophen glucuronide-nTMS   1", "Cholesta-3,5-diene   1", "GAMMA-TOCOPHEROL-TMS   1", "1-PALMITOYL-sn-GLYCERO-3-PHOSPHOCHOLINE-TMS derivative   1", "ALPHA-TOCOPHEROL-TMS   1", "CHOLESTEROL-TMS   1", "1-STEAROYL-sn-GLYCERO-3-PHOSPHOCHOLINE-TMS derivative   1", "M001155_A329007-101-xxx_NA_3298,55_TRUE_VAR5_ALK_Lithocholic acid (2TMS)   1", "W314_N/A like Fatty acid   1", "MALTOTRIOSE-MEOX 11TMS_1   1", "W353_Phenol, 2,4-bis(1,1-dimethylethyl)-, phosphite (3i1)   1", "W316_Tris(2,4-di-tert-butylphenyl) phosphate   1", "W251_N/A like Fatty acid   1", "W351_N/A like Fatty acid   1", "Leucine_RDA", "Isocitric acid_RDA", "Citric acid_RDA"
    ]
    # THIS IS A WASTEFUL ALGORITHM THAT WORKS WHEN YOU HAVE ALOT OF RAM
    # FIRST DOWNLOAD DATABASE FROM http://www.hmdb.ca/downloads AND PUT IN data/ DIR
    #
    tree = etree.parse('data/serum_metabolites.xml')
    root = tree.getroot()

    #translate_these = ['2-Ketobutyric acid','(2S)-2-amino-3-(1-Methyl-1H-imidazol-4-yl)propanoate','lysoPC(28:1(5Z))']
    check = set(translate_these)
    translation = {}
    acc = []
    for child in root:
        acc,nam,cey = get_accession( child, common_name=translate_these )
        if len(acc)>0:
            translation[cey]=acc[0]
            check-=set(cey)
        if len(check)==0:
            break
    print(translation)

    """
    # RESULTS IN > 

{'Chenodeoxycholic acid': 'HMDB0000518', 'Cholic acid': 'HMDB0000619', 'Chenodeoxycholic acid glycine conjugate': 'HMDB0000637', 'Methylsuccinic acid': 'HMDB0001844', 'Lithocholic acid glycine conjugate': 'HMDB0000698', 'Drospirenone': 'HMDB0015467', '3-Hydroxy-2-ethylpropionate': 'HMDB0000396', '3-Hydroxyoctanoate': 'HMDB0001954', 'Glycoursodeoxycholic acid': 'HMDB0000708', '3-Hydroxydecanoate': 'HMDB0002203', 'Ethylmalonic acid': 'HMDB0000622', 'Taurodeoxycholic acid': 'HMDB0000896', '2-Methylglutaric acid': 'HMDB0000422', 'Glycocholic acid': 'HMDB0000138', 'gamma-CEHC': 'HMDB0001931', '1-Arachidonoylglycerol': 'HMDB0011578', 'EDTA': 'HMDB0015109', 'Betaine': 'HMDB0000043', 'Carnitine': 'HMDB0062496', '3,7-Dihydroxy-5-cholestenoic acid': 'HMDB0012454', 'Ketoleucine': 'HMDB0000695', '3-Methylcrotonylglycine': 'HMDB0000459', 'Taurochenodesoxycholic acid': 'HMDB0000951', '3-Hydroxylaurate': 'HMDB0000387'}
    """
